from smbus2     import SMBus, i2c_msg
from time       import sleep
import numpy    as np
import ov5647   as ov

I2C_SADR_OV5647         = 0x36
I2C_SADR_TC358748       = 0x0e


def cis_i2c_write( reg_addr, data ):
    command     = [reg_addr >> 8, reg_addr & 0xff, data]
    write       = i2c_msg.write( I2C_SADR_OV5647, command )

    with SMBus(1) as i2c:
        i2c.i2c_rdwr( write )
        print( 'Register Write OV5647   : addr=0x%04x data=0x%02x' %(reg_addr, data) )


def cis_i2c_read( reg_addr ):
    command     = [reg_addr >> 8, reg_addr & 0xff]
    write       = i2c_msg.write( I2C_SADR_OV5647, command )
    read        = i2c_msg.read ( I2C_SADR_OV5647, 1 )

    with SMBus(1) as i2c:
        i2c.i2c_rdwr( write, read )
        data        = list( read )
        print( 'Register Read  OV5647   : addr=0x%04x data=0x%02x' %(reg_addr, data[0]) )


def m2p_i2c_write( reg_addr, data ):
    command     = [reg_addr >> 8, reg_addr & 0xff, data >> 8, data & 0xff]
    write       = i2c_msg.write( I2C_SADR_TC358748, command )

    with SMBus(1) as i2c:
        i2c.i2c_rdwr( write )
        print( 'Register Write TC358748 : addr=0x%04x data=0x%04x' %(reg_addr, data) )


def m2p_i2c_read( reg_addr ):
    command     = [reg_addr >> 8, reg_addr & 0xff]
    write       = i2c_msg.write( I2C_SADR_TC358748, command )
    read        = i2c_msg.read ( I2C_SADR_TC358748, 2 )

    with SMBus(1) as i2c:
        i2c.i2c_rdwr( write, read )
        data        = list( read )
        print( 'Register Read  TC358748 : addr=0x%04x data=0x%02x%02x' %(reg_addr, data[0], data[1]) )


def cis_init( video_res, data_mode ) :
    print( '### OV5647    Initialize start -->' )

    cis_i2c_write( 0x3000, 0x0f )
    cis_i2c_write( 0x3001, 0xff )
    cis_i2c_write( 0x3002, 0xe4 )

    if      video_res   == "1080P":
        for name in ov.mode_1920x1080:
            cis_i2c_write( name['adrs'], name['data'] )
    elif    video_res   == "480P":
        for name in ov.mode_640x480:
            cis_i2c_write( name['adrs'], name['data'] )

    print( '--> OV5647 Initialize end ###' )


def m2p_init():
    print( '### TC358748 Initialize start -->' )

    print( 'TC358748 Software Reset' )
    m2p_i2c_write( 0x0002, 0x0001 )         # SYSctl, S/W Reset
    sleep( 0.0001 )
    m2p_i2c_write( 0x0002, 0x0000 )         # SYSctl, S/W Reset release

    print( 'TC358748 PLL,Clock Setting' )
    m2p_i2c_write( 0x0016, 0x0015 )         # PLL Control Register 0 (PLL_PRD,PLL_FBD)
    m2p_i2c_write( 0x0018, 0x0403 )         # PLL_FRS,PLL_LBWS, PLL oscillation enable
    sleep( 0.001 )
    m2p_i2c_write( 0x0018, 0x0413 )         # PLL_FRS,PLL_LBWS, PLL clock out enable
    m2p_i2c_write( 0x0020, 0x0011 )         # CLK control register: Clock divider setting
    m2p_i2c_write( 0x000C, 0x0101 )         # MCLK duty setting

    print( 'TC358748 Format configuration, timing Setting' )
    m2p_i2c_write( 0x0060, 0x8009 )         # MIPI PHY Time Delay

    m2p_i2c_write( 0x0006, 0x0010 )         # FIFO control
    m2p_i2c_write( 0x0008, 0x0010 )         # Data format control
    m2p_i2c_write( 0x0004, 0x004d )         # Configuration control

    print( '--> TC358748 Initialize end ###' )


def cis_start() :
    print( '### OV5647 Streaming -->' )

    cis_i2c_write( 0x4800, 0x04 )
    cis_i2c_write( 0x4202, 0x00 )
    cis_i2c_write( 0x300D, 0x00 )
    sleep( 0.001 )

#    cis_i2c_write( 0x503D, 0x80 )
    

def cis_stop() :
    print( '--> OV5647 Standby ###' )
    
    cis_i2c_write( 0x4800, 0x25 )
    cis_i2c_write( 0x4202, 0x0f )
    cis_i2c_write( 0x300D, 0x01 )
    sleep( 0.001 )

    cis_i2c_write( 0x3000, 0x00 )
    cis_i2c_write( 0x3001, 0x00 )
    cis_i2c_write( 0x3002, 0x00 )
